from blenderline.collections.base import BaseCollection
from blenderline.entries.hdr import HDREntry


class HDRCollection(BaseCollection[HDREntry]):
    """Collection of registered HDR background entries."""

    pass
