from blenderline.collections.base import BaseCollection
from blenderline.entries.item import ItemEntry


class ItemCollection(BaseCollection[ItemEntry]):
    """Collection of registered item entries."""

    pass
