from blenderline.collections.base import BaseCollection
from blenderline.entries.background import BackgroundEntry


class BackgroundCollection(BaseCollection[BackgroundEntry]):
    """Collection of registered background entries."""

    pass
